


 //
 // RFPCK.C: Created 2003 01 21
 //    RF-chip procedures for tx'ing/rx'ing "packets" with a defined format
 //
 // (C) Micrel Norway AS
 //     Proprietary firmware of Micrel Norway AS
 //
 


 // Note: The intention of the firmware listed below is to help users 
 //       get hands-on experience with Micrel Norway's products.
 
 //       No guarantee is implied as to the suitableness of the firmware 
 //       in a given application.
 
 //       For further information, please contact post@micrel.no



 // Changes:
 //   2004 04 05 PKB
 //      Select Manchester code by #define MANCH_CODE
 //         if not def'ed: 1:1 code is used
 //      rx-regs: changed from byte3,2,1,0 to 0,1,2 and def'ed sample as lsb
 // 
 //   2004 02 05 PKB:
 //      only data (not length/crc stored in buffers)
 //      not changed: "length" is for length-field + data-field + crc-field
 //      not changed: length is included in crc
 //      "RF_ACTIVE" is not set/cleared when rxing (rx is stopped if user wants to tx)




  #define C_FILE     // Tested in "p18xxxx.h" (to include correct file for C/ASM - programs)
  #define RFPCK_FILE // The file "RFPCK.h" tests if this definition is made or not

  #define MANCH_CODE // Define if using Manchester code. Undefine if using 1:1 code
 
 

//*************************************************
//
// Include files
//
//************************************************* 
 #include "p18def.h"   // Define PIC used
 #include "p18xxxx.h"  // Standard Microchip file for the selected PIC
 #include "c_subst.h"  // Commonly used C - definitions
 
 #include "rfpck.h"    // "RF packet" definitions
 #include "rfprg.h"    // "RF programming" defs
 #include "io_xxx.h"   // General I/O file which includes the application-specific I/O
 #include "crc.h"      //
 #include "gen.h"      // General defs
 #include "usart.h"    //


// Future: Include these defs in a #include
 extern volatile near uint8
   TIM_Cap_PC,
   TIM_Cap_Timer2h,
   TIM_Cap_Timer2u,
   TIM_Period_Counter,
   TIM_Timer2h,
   TIM_Timer2u,
   TIM_1_1, // "capture set 1"
   TIM_1_2,
   TIM_1_3,
   TIM_2_1, // "capture set 2"
   TIM_2_2,
   TIM_2_3;


 extern volatile near union {
             struct {
                uint8 all;
             };
             struct {
                unsigned bit0:1;
                unsigned bit1:1;
                unsigned bit2:1;
                unsigned bit3:1;
                unsigned bit4:1;                
                unsigned bit5:1;
                unsigned bit6:1;
                unsigned bit7:1;
             };
          } TIM_Flags;

   #define TIM_SET1_OK    TIM_Flags.bit4  



//*************************************************
//
// Code starts here
//
//************************************************* 
 #pragma code
 #pragma udata
 
 
void
RFPCK_Enable_RFTX_Interrupt( void)
{   
   INTCON2bits.INTEDG0 = 1; // Int on raising edge 
                            // RF chip samples DATAIXO on raising edge of DATACLK- 
                            // that is, when a raising edge is detected, 
                            // the dataline is sampled and ready to be changed
                            // if using spi-like interface: make sure to clock in bytes on falling edges of clock

   INTCONbits.INT0IF   = 0; // Clear int flag
   INTCONbits.INT0IE   = 1; // Enable interrupt
}



void
RFPCK_Enable_RFRX_Interrupt( void)
{   
   INTCON2bits.INTEDG0 = 0; // Int on falling edge 
                            // RF chip changes DATAIXO on raising edge of DATACLK- 
                            // that is, when a raising edge is detected, 
                            // the dataline is just changed
                            // Therefore, sample DATAIXO a "long time" after the raising edge, 
                            // for example on the falling edges

   INTCONbits.INT0IF   = 0; // Clear int flag
   INTCONbits.INT0IE   = 1; // Enable interrupt
}


void
RFPCK_Disable_RF_Interrupt( void)
{   
   INTCONbits.INT0IE   = 0; // Disable interrupt
   INTCONbits.INT0IF   = 0; // Clear int flag   
}


// Purpose: Initiate for rxing of a packet
void
RFPCK_Start_Rx_Packet( void)
{
   RFPCK_Disable_RF_Interrupt();   
   RFPRG_Enter_Rx();

   if( not RFPCK_Rx_Flags.PCK_FOUND)   // If already got a packet, then do not start searching
   {      
      RFPCK_Init_Rx();
      RFPCK_Rx_Flags.RX_STARTED = 1;
   }
}


void
RFPCK_Init_Rx( void)
{
   RFPCK_Tx_Flags.all = 0x00;
   RFPCK_Rx_Flags.all = 0x00;

   RFPCK_Rxregs.byte0 = 0x00;
 
   RFPCK_Enable_RFRX_Interrupt();
}


void
RFPCK_Rx_Int_Handler( void)
{
   if( not RFPCK_Rx_Flags.PCK_FOUND)  // Continue only if not already received a packet
   {        
    //if( not RFPCK_Rx_Flags.RX1010_TEST) // If rx1010 test, then don't search for sync etc
    //{      
         LED4 = 0; //!!!!
         if( not RFPCK_Rx_Flags.SYNC_FOUND) // Test if sync is already found or not
         {         
            if( ( RFPCK_Rxregs.byte0 == (uint8)RFPCK_SYNCELEM) and  // Test if sync is found now
                ( RFPCK_Rxregs.byte1 == (uint8)RFPCK_SYNCELEM) and
                ( RFPCK_Rxregs.byte2 == (uint8)RFPCK_SYNCELEM) )
            { 
               RFPCK_Rx_Flags.SYNC_FOUND = 1;            
             //RFPCK_Status2.RF_ACTIVE   = 1;
                        
               RFPCK_Rxregs.byte0 = 0x00;
            
               FCS.H = 0x00;
               FCS.L = 0x00;           
               RFPCK_Elmncntr.all = 0x00;

               //Note: Since this is a high-pri-int it can interrupt a low-pri interrupt
               // make sure: If TIM-regs started to be changed, then make sure they are changed completely
               if( TIM_SET1_OK)
               {
                  TIM_Cap_PC      = TIM_1_1;
                  TIM_Cap_Timer2h = TIM_1_2;
                  TIM_Cap_Timer2u = TIM_1_3;
               }
               else
               {
                  TIM_Cap_PC      = TIM_2_1;
                  TIM_Cap_Timer2h = TIM_2_2;
                  TIM_Cap_Timer2u = TIM_2_3;
               }
            }      
         }
         else
         {
            RFPCK_Elmncntr.all++;

            #ifdef MANCH_CODE
            if( not RFPCK_Elmncntr.bit0) // Test if read 2 elements = 1 bit
            {
               RFPCK_Temp <<= 1;
               if( RFPCK_Rxregs.sample) // assumes 1 is coded as "01" and 0 is coded as "10"
               {
                  RFPCK_Temp++;

                  FCS.Present = 0xFF;
                  FCS.all <<= 1;
                  if( STATUSbits.C)
                  {
                     FCS.L ^= 0x21;
                     FCS.H ^= 0x10;
                  }
               }
               else
               {
                  FCS.Present = 0x00;
                  FCS.all <<= 1;
                  if( STATUSbits.C)
                  {
                     FCS.L ^= 0x21;
                     FCS.H ^= 0x10;
                  }
               }
            }
            if( RFPCK_Elmncntr.bit4) // Test if read 16 elements = 8 bits
            {
               //FCS.Present = RFPCK_Temp; // Do this on a bit-by-bit basis?
               //DEBUG!!!!!CRC_Byte();               //
            #endif

            #ifndef MANCH_CODE            
            RFPCK_Temp <<= 1;
            if( RFPCK_Rxregs.sample) // assumes 1 is coded as "01" and 0 is coded as "10"
            {
               RFPCK_Temp++;

               FCS.Present = 0xFF;
               FCS.all <<= 1;
               if( STATUSbits.C)
               {
                  FCS.L ^= 0x21;
                  FCS.H ^= 0x10;
               }
            }
            else
            {
               FCS.Present = 0x00;
               FCS.all <<= 1;
               if( STATUSbits.C)
               {
                  FCS.L ^= 0x21;
                  FCS.H ^= 0x10;
               }
            }
            if( RFPCK_Elmncntr.bit3) // Test if read 8 bits
            {         
               //RFPCK_Temp = RFPCK_Rxregs.byte0; 
               //FCS.Present = RFPCK_Temp; // Do this on a bit-by-bit basis?
               //CRC_Byte(); 
            #endif


            
               RFPCK_Elmncntr.all = 0x00;
               
               if( not RFPCK_Rx_Flags.LENGTH_FOUND)
               {
                  RFPCK_Temp_Length = RFPCK_Temp - 1; // length of payload level 1 + crc 
                  RFPCK_In_Length   = RFPCK_Temp - 3; // length of payload level 1 (subtract 3 (#length and #crc bytes) to get  payload length)
                  if( ( RFPCK_In_Length > (uint8)RFPCK_BUFFERSIZE) or ( RFPCK_In_Length == (uint8)0))
                  {
                     RFPCK_Rx_Flags.SYNC_FOUND = 0; // Not a legal length field, ignore
                   //RFPCK_Status2.RF_ACTIVE   = 0; // !!!!! !!!!!
                  }
                  else
                  {
                     RFPCK_Rx_Flags.LENGTH_FOUND = 1;  // Length is OK, keep it and start reading bytes
                     RFPCK_Temp_Pos = RFPCK_In_Nextin; // Store location where 1st databyte will be stored
                  }
               }
               else
               {   
                  if( RFPCK_Temp_Length > (uint8)2) // Do not store crc in buffer
                  {
                     RFPCK_In_Buffer[ RFPCK_Temp_Pos] = RFPCK_Temp; // Store correctly
     
                     if( RFPCK_Temp_Pos == (uint8)(RFPCK_BUFFERSIZE-1))
                     {
                        RFPCK_Temp_Pos = 0;
                     }
                     else
                     {
                        RFPCK_Temp_Pos++;
                     }
                  }
  
                  RFPCK_Temp_Length--;       //RFPCK_Rf_In_Length--;
                  if( not RFPCK_Temp_Length) //RFPCK_Rf_In_Length)   // Test if all bytes in packet is read or not
                  {
                   //GEN_Reset_Tmr1_Ccpx();  // Stop timer/ccp modules and disable interrupt
                     RFPCK_Rx_Flags.SYNC_FOUND   = 0;                                       
                     RFPCK_Rx_Flags.LENGTH_FOUND = 0;
                   //LED3 = 1;
                   //LED3 = 0;
                     if( ( not FCS.H) and ( not FCS.L) ) // Test if FCS is OK
                     {
                      //LED4 = 1; //!!!!!
                      //LED4 = 0; //!!!!!
                        RFPCK_Rx_Flags.PCK_FOUND = 1;     // Must be detected by main program
                        RFPCK_First_Byte_Pos = RFPCK_In_Nextin;
                        RFPCK_In_Nextin = RFPCK_Temp_Pos; // Data accepted, update in-pointer
                     }
                     
                  }                  

               } // end length read/not read
            } // end read a byte
         } // end sync found/not found
    //} // end rx1010 test
   } // end pck found
   RFPCK_Rxregs.all <<= 1;  // Read msb first.  Default: Next sample of datainput is "0"
}
       




/* !!!!! Note: To avoid setting "0" or "1" at different timer-values */
/* !!!!! (difference is 1 instruction cycle):                        */
/* !!!!! Due to compiler. Including a NOP to equate #instr           */
void
RFPCK_Tx_Int_Handler( void)
{
   static uint8 bit_index;
   
      #ifdef MANCH_CODE
      if( RFPCK_Tx_Flags.DO_MANCHESTER)
      {
         if( not RFPCK_Tx_Flags.ELEMENT1_DONE)
         {
            if( not RFPCK_Txreg1.bit7) //!!!!! msb first!!!!!, txing "10" for "0" and "01" for "1"
            { 
               Nop(); // Equate number of instr cycles
               RF_DATA_OUT = 1;
            } 
            else
            {  
                RF_DATA_OUT = 0;
            }
            RFPCK_Tx_Flags.ELEMENT1_DONE = 1;
         }
         else
         {
            RF_DATA_OUT = !RF_DATA_OUT ;
            RFPCK_Tx_Flags.ELEMENT1_DONE = 0;
            RFPCK_Txreg1.all <<= 1;
            RFPCK_Elmncntr.all++; // Note: Inititated to 0x00
         }
      }
      else
      {
         if( RFPCK_Txreg1.bit7) //!!!!! msb first!!!!!
         { 
            RF_DATA_TRIS = 0; //Nop(); // Equate number of instr cycles and make sure dataline is output!
            RF_DATA_OUT = 1;
         }
         else
         { 
            RF_DATA_OUT = 0;
         }
         RFPCK_Txreg1.all <<= 1;
         RFPCK_Elmncntr.all++; // Note: Inititated to 0x00
      }
      #endif


      #ifndef MANCH_CODE
      if( RFPCK_Txreg1.bit7) //!!!!! msb first!!!!!
      { 
         RF_DATA_TRIS = 0; //Nop(); // Equate number of instr cycles and make sure dataline is output!
         RF_DATA_OUT  = 1;
      }
      else
      { 
          RF_DATA_OUT = 0;
      }
      RFPCK_Txreg1.all <<= 1;
      RFPCK_Elmncntr.all++; // Note: Inititated to 0x00
      #endif


      if( RFPCK_Elmncntr.bit3) // Test if txed a complete 8-element pattern (in fact: just started to tx last element)
      {
         RFPCK_Txreg1.all   = RFPCK_Txreg0.all;
         RFPCK_Elmncntr.all = 0x00;

         switch( RFPCK_Rf_State)
         {
            case 0:
          //case 1: //reduced preamble to 24 elements
          //case 2:
          //case 3:
               RFPCK_Txreg0.all = RFPCK_CLKELEM;
             //if( not RFPCK_Tx_Flags.TX1010_TEST) // if tx1010 test, then keep txing 101010...
             //{
                  RFPCK_Rf_State = 4; //++;
             //}
               break;
   
            case 4:
            case 5:
            case 6:
               RFPCK_Txreg0.all = RFPCK_SYNCELEM;
               RFPCK_Rf_State++;
               break;

            case 7: // When entered: Start to tx the last sync_elements in next interrupt
               RFPCK_Txreg0.all = ( RFPCK_Temp_Length + 3);
               RFPCK_Rf_State++;            
               break;
            
            case 8: // When entered: Start to tx the length-field in next interrupt
               //Note: Since this is a high-pri-int it can interrupt a low-pri interrupt
               // make sure: If TIM-regs started to be changed, then make sure they are changed completely
               if( TIM_SET1_OK)
               {
                  TIM_Cap_PC      = TIM_1_1;
                  TIM_Cap_Timer2h = TIM_1_2;
                  TIM_Cap_Timer2u = TIM_1_3;
               }
               else
               {
                  TIM_Cap_PC      = TIM_2_1;
                  TIM_Cap_Timer2h = TIM_2_2;
                  TIM_Cap_Timer2u = TIM_2_3;
               }
               CRC_Flags.GOT_VALUE = 1; // Do not wait any longer (if crc routine was waiting for these bytes)

               #ifdef MANCH_CODE
               RFPCK_Tx_Flags.DO_MANCHESTER = 1;
               #endif
               RFPCK_Rf_State++;
             //break; // Note: No break from this case

            case 9:
               RFPCK_Txreg0.all = RFPCK_Out_Buffer[ RFPCK_Temp_Pos]; 
               RFPCK_Temp_Length--;

               if( RFPCK_Temp_Pos == (uint8)( RFPCK_BUFFERSIZE - 1))
               {
                  RFPCK_Temp_Pos = 0;
               }
               else
               {
                  RFPCK_Temp_Pos++;
               }         

               if( not RFPCK_Temp_Length)
               {
                  RFPCK_Rf_State++;
               }                        

               break;

            case 10:
               RFPCK_Txreg0.all = FCS.H;
               RFPCK_Rf_State++;
               break;


            case 11:
               RFPCK_Txreg0.all = FCS.L;
               RFPCK_Rf_State++;
               break;


            case 12: // When entered: Start to tx fcs_l in next interrupt
               RFPCK_Txreg0.all = RFPCK_DUMMYELEM;
               RFPCK_Rf_State++;
               break;

            case 13: // When entered: fcs_l txed completely, wait for last element to end before returning
               RFPCK_Txreg0.all = RFPCK_DUMMYELEM;
             //RFPCK_Tx_Flags.TXREG0_EMPTY = 1;
               RFPCK_Elmncntr.all = 0x07; // Make sure entered into state-machine at next interrupt
               RFPCK_Rf_State++;
               break;

            default: //case 14: (not nescassary to test for this case - increase speed)
             //RFPCK_Tx_Flags.TX_FINISHED = 1; // Must be detected in main program
               RFPCK_Tx_Flags.TX_STARTED  = 0;
               RFPCK_Out_Nextout = RFPCK_Temp_Pos;
             //GEN_Reset_Tmr1_Ccpx();        // Stop timer/ccp modules and disable interrupt
               RFPCK_Disable_RF_Interrupt();
             //RF_DATA_TRIS = 1; //Set dataline as input again
               LED3 = 0; //!!!!!
               break;
         }

      } // end txed a complete 8-element byte
}



// Purpose: Initiate for txing of a "packet"
// Pre:   
//        RFPCK_Out_Length holds number of data bytes
//        RFPCK_Out_Buffer holds the bytes to tx
//        RFPCK_Out_Nextout holds pos of 1st byte in buffer to tx
void
RFPCK_Start_Tx_Packet( void)
{
   if( not RFPCK_Tx_Flags.TX_STARTED) // If already started, then do not start a new one
   {
      RFPCK_Disable_RF_Interrupt();            
      RFPRG_Enter_Tx();      

      RFPCK_Temp_Pos    = RFPCK_Out_Nextout;
      RFPCK_Temp_Length = RFPCK_Out_Length;

      RFPCK_Init_Tx();
      
      RFPCK_Tx_Flags.TX_STARTED  = 1;
    //RFPCK_Rx_Flags.RX_STARTED  = 0;
    //RFPCK_Tx_Flags.TX_FINISHED = 0;      
    //RFPCK_Status2.RF_ACTIVE    = 1;
   }
}


// Pre: RF transceiver is in transmitmode
void
RFPCK_Init_Tx( void)
{
 //RFPCK_Disable_RF_Interrupt();

   RFPCK_Tx_Flags.all = 0x00;
   RFPCK_Rx_Flags.all = 0x00;

   RFPCK_Tx_Flags.TXING = 1;

   RFPCK_Txreg0.all = RFPCK_CLKELEM; // Initate transmission registers
   RFPCK_Txreg1.all = RFPCK_CLKELEM; //  with "clock elements"

   RFPCK_Elmncntr.all = 0x00; // Initiated to 0x00

   RF_DATA_OUT  = 1; // The first element txed is "1"
 //RF_DATA_TRIS = 0; // Set as output in ISR
   
 //RFPCK_Rf_Out_Pos = 0;
   RFPCK_Rf_State   = 0;   
   
   RFPCK_Enable_RFTX_Interrupt();
}






void
RFPCK_Init_Regs( void)
{
   RFPCK_In_Nextin   = 0x00;
   RFPCK_In_Nextout  = 0x00;
   
   RFPCK_Out_Nextin  = 0x00;
   RFPCK_Out_Nextout = 0x00;
   
   RFPCK_In_Length  = 0x00;
   RFPCK_Out_Length = 0x00;

   RFPCK_Tx_Flags.all  = 0x00;
   RFPCK_Rx_Flags.all  = 0x00;
}



void
RFPCK_Update_Out_Nextin( void)
{
   if( RFPCK_Out_Nextin == (uint8)(RFPCK_BUFFERSIZE - 1))
   {
      RFPCK_Out_Nextin = 0;
   }
   else
   {
      RFPCK_Out_Nextin++;
   }
}


      
void
RFPCK_Update_In_Nextout( void)         
{
   if( RFPCK_In_Nextout == (uint8)(RFPCK_BUFFERSIZE - 1))
   {
      RFPCK_In_Nextout = 0;
   }
   else
   {
      RFPCK_In_Nextout++;
   }	     
}
