


 //
 // USART.C: Created 2003 03 18
 //    Usart procedures
 //
 // (C) Micrel Norway AS
 //     Proprietary firmware of Micrel Norway AS
 //
 


 // Note: The intention of the firmware listed below is to help users 
 //       get hands-on experience with Micrel Norway's products.
 
 //       No guarantee is implied as to the suitableness of the firmware 
 //       in a given application.
 
 //       For further information, please contact post@micrel.no




 #define C_FILE     // Tested in "p18xxxx.h" (to include correct file for C/ASM - programs)
 #define USART_FILE //


//*************************************************
//
// Include files
//
//************************************************* 
 #include "p18def.h"  // Define PIC used
 #include "p18xxxx.h" // Standard Microchip file for the selected PIC
 #include "c_subst.h" //
 
 #include "io_xxx.h"  //
 #include "usart.h"   // Prototypes, variables and constants
 #include "gen.h"     // 


//*************************************************
//
// Code starts here
//
//************************************************* 
 #pragma code
 #pragma udata

void
USART_Output_Byte( uint8 usart_byte)
{

   while( not PIR1bits.TXIF)
   {
   }
   TXREG = usart_byte;
   
}



void
USART_Init( void)
{
	
   PIE1bits.RCIE = 0;

//ext xtal:11.0592MHz

   RCSTA = 0x00;
   TXSTA = 0x00;
   
   SPBRG = 71; // Baud rate:   9600, xtal 11.0592MHz, high speed
 //SPBRG = 25; // Baud rate    2400, xtal 4MHz, low speed 
   
   TXSTA = 0b00100100; // High speed
 //TXSTA = 0b00100000; // Low speed
      
 // Bit7: CSRC Don't care when SYNC=0
 // Bit6: TX9  = 0 -> 8 bit transmission
 // Bit5: TXEN = 1 -> Transmit enable
 // Bit4: SYNC = 0 -> Asynchronous mode
 // Bit3: Unimplemented
 // Bit2: BRGH = 1 -> High speed
 // Bit1: TRMT this bit is read-only
 // Bit0: TX9D Don't care when TX9=0

   RCSTA = 0b10000000;
   RCSTA = 0b10010000;
   
 // Bit7: SPEN = 1 -> Serial port enabled
 // Bit6: RX9  = 0 -> 8-bit reception
 // Bit5: SREN Don't care when SYNC=0
 // Bit4: CREN = 1 -> Enable reception
 // Bit3: Unimplemented
 // Bit2: FERR this bit is read-only
 // Bit1: OERR this bit is read-only
 // Bit0: RX9D this bit is read-only
 
   USART_Flags.all   = 0x00;

   USART_Out_Nextin  = 0x00;
   USART_Out_Nextout = 0x00;  
   USART_Out_Length  = 0x00;
  
   USART_In_Nextin   = 0x00;
   USART_In_Nextout  = 0x00;
   USART_In_Length   = 0x00;    
  
   IPR1bits.RCIP = 0;
   PIE1bits.RCIE = 1;
   
}
	


// Pre: USART_Out_Length holds number of bytes to give out
//      USART_Out_Nextout points to next byte to give out
//      (i.e. it holds the index of the buffer)
//
void
USART_Start_Buffer_Out( void)
{
   //if( ( not USART_Flags.USART_OUT_ACTIVE) and ( USART_Out_Length > (uint8)0) )
   //{
    //USART_Flags.USART_OUT_ACTIVE = 1;
      IPR1bits.TXIP = 0; // Low pri
    //PIR1bits.TXIF = 0; // Clear int flag (done when loading TXREG)
      PIE1bits.TXIE = 1; // Enable tx-interrupt
   //}     
   
}



void
USART_TX_Int_Handler( void)
{
   if( USART_DTE_Ready_For_Input) // Is this sufficient?????
   {
    //PIR1bits.TXIF = 0;   // Clear int flag (done when loading TXREG)
      TXREG = USART_Out_Buffer[ USART_Out_Nextout];
   
      if( USART_Out_Nextout == (uint8)(USART_BUFFERSIZE - 1) )
      {
         USART_Out_Nextout = 0x00;
      }
      else
      {
         USART_Out_Nextout++;
      }
   
      USART_Out_Length--;

      if( not USART_Out_Length)
      {
         PIE1bits.TXIE = 0; // Disable tx-interrupt
       //USART_Flags.USART_OUT_ACTIVE = 0;
      }
   }
   
}



void
USART_RX_Int_Handler( void)
{
   if( USART_In_Length == (uint8)USART_BUFFERSIZE)
   {
      WREG = RCREG; // This byte is lost; must read it to avoid overflow
   }
   else
   {
      USART_In_Buffer[ USART_In_Nextin] = RCREG;
            
      if( USART_In_Nextin == (uint8)(USART_BUFFERSIZE - 1) )
      {
         USART_In_Nextin = 0x00;
      }
      else
      {
         USART_In_Nextin++;
      }
      
      USART_In_Length++;

      if( USART_In_Length >= (uint8)USART_MAX_FROM_DTE)
      {
         USART_Disable_Input_From_DTE();
         //set flag for got user input?
      }
   }
 
      
 //T0CON = 0x87; // TMR0 enabled, 16-bit, int instr, pre 1:256
 //TMR0H = 0xF3;
 //TMR0L = 0x58; 
 //INTCONbits.TMR0IF = 0; // In main: If got inputbytes, and flag set, then set flag for got-input
   
   
   //T1CON = 0;
   //TMR1  = 63808; // 2^16-1728 ( 1728*8*4/11.0592e6 => 5 msec)
   //PIR1bits.TMR1IF = 0;
   //T1CON = 0x31;  // TMR1 enabled, pre 1:8   
}



void
USART_Enable_Input_From_DTE( void)
{
	
   if( not USART_Flags.SKIP_HS)
   {
      CTS_OUTPUT = 0;
   }
   
}


void
USART_Disable_Input_From_DTE( void)
{
	
   if( not USART_Flags.SKIP_HS)
   {
      CTS_OUTPUT = 1;
   }
   
}


// Return "true" if DTE is ready for input, else return "false"
uint8
USART_DTE_Ready_For_Input( void)
{
	
   if( USART_Flags.SKIP_HS)
   {
      return( 0xFF); //Ready    
   }
   
   if( not RTS_INPUT)
   {
      return( 0xFF); //Ready
   }
   else
   {
      return( 0x00); // Not ready
   }
   
}

 
